/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Hashtable;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import ch.cern.lhcb.xmleditor.specific.XmlDisplay;

/**
 * A dialog windows that allows configuration of attribute displaying in an
 * XmlTree
 * 
 * @author Sebastien Ponce
 */
public class ElementDialog extends AbstractDialog implements ItemListener {

    /**
     * the layout manager of the panel
     */
    private GridBagLayout layout;

    /**
     * a hashtable of subelement names that currently have an associated
     * checkbox indicating if they are displayed in the tree or not, indexed
     * through their checkbox
     */
    private Hashtable displayInTreeBoxes;

    /**
     * a hashtable of element names that currently have an associated checkbox
     * indicating if they are displayed as attributes or not, indexed through
     * their checkbox
     */
    private Hashtable displayAsAttrBoxes;

    /**
     * constructor : builds the dialog
     * 
     * @param owner the frame where this dialog should appear
     * @param xmlDisplay the XmlDisplay that must be used
     */
    public ElementDialog(Frame owner, XmlDisplay xmlDisplay) {
        // calls the super constructor
        super(owner, "Subelement displaying", xmlDisplay);
        // initializes the hashtables
        this.displayInTreeBoxes = new Hashtable();
        this.displayAsAttrBoxes = new Hashtable();
        // adds a layout to the panel where details are displayed
        layout = new GridBagLayout();
        panel.setLayout(layout);
    }

    /**
     * builds the panel for a given element name of a given XmlDisplay by
     * displaying some subelements
     * 
     * @param name the name of the element for which the panel should be built
     * @param xmlDisplay the XmlDisplay dealing with the dtd in which the
     *        element name was declared
     */
    protected void buildPanel(String name, XmlDisplay xmlDisplay) {
        // purge the hashtables
        displayInTreeBoxes.clear();
        displayAsAttrBoxes.clear();
        // stores the current xmlDisplay
        this.xmlDisplay = xmlDisplay;
        // retrieves the subelements
        String[] subElements = xmlDisplay.getSubElementNames(name);
        // creates a GridBagConstraint object that will be used for display
        GridBagConstraints c = new GridBagConstraints();
        // makes titles for each column
        JLabel elemLabel = new JLabel("Subelement's Name");
        Font font = new Font("dialog", Font.PLAIN, 16);
        elemLabel.setFont(font);
        elemLabel.setAlignmentX(SwingConstants.LEFT);
        c.weightx = 1;
        c.gridx = 0;
        c.gridy = 1;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(0, 0, 5, 0);
        layout.setConstraints(elemLabel, c);
        panel.add(elemLabel);
        JLabel cbLabel1 = new JLabel("displayed");
        JLabel cbLabel2 = new JLabel("in tree");
        cbLabel1.setFont(font);
        cbLabel1.setAlignmentX(SwingConstants.CENTER);
        cbLabel2.setFont(font);
        cbLabel2.setAlignmentX(SwingConstants.CENTER);
        c.weightx = 0;
        c.gridx = 1;
        c.gridy = 0;
        c.anchor = GridBagConstraints.CENTER;
        layout.setConstraints(cbLabel1, c);
        c.gridy = 1;
        layout.setConstraints(cbLabel2, c);
        panel.add(cbLabel1);
        panel.add(cbLabel2);
        JLabel cbLabel3 = new JLabel("displayed");
        JLabel cbLabel4 = new JLabel("as attribute");
        cbLabel3.setFont(font);
        cbLabel3.setAlignmentX(SwingConstants.CENTER);
        cbLabel4.setFont(font);
        cbLabel4.setAlignmentX(SwingConstants.CENTER);
        c.weightx = 0;
        c.gridx = 2;
        c.gridy = 0;
        c.anchor = GridBagConstraints.CENTER;
        c.insets = new Insets(0, 10, 5, 0);
        layout.setConstraints(cbLabel3, c);
        c.gridy = 1;
        layout.setConstraints(cbLabel4, c);
        panel.add(cbLabel3);
        panel.add(cbLabel4);
        // for each subelement, displays it
        for (int i = 0; i < subElements.length; i++) {
            // creates and display label
            JLabel label = new JLabel(subElements[i]);
            label.setAlignmentX(SwingConstants.LEFT);
            c.weightx = 1;
            c.gridx = 0;
            c.gridy = i + 2;
            c.anchor = GridBagConstraints.WEST;
            c.insets = new Insets(0, 10, 0, 0);
            layout.setConstraints(label, c);
            panel.add(label);
            // creates and display checkbox for displaying in tree
            JCheckBox checkBox = new JCheckBox();
            displayInTreeBoxes.put(checkBox, subElements[i]);
            checkBox.setSelected(xmlDisplay.isSubElementDisplayed(name,
                    subElements[i]));
            checkBox.addItemListener(this);
            c.weightx = 0;
            c.gridx = 1;
            c.anchor = GridBagConstraints.CENTER;
            layout.setConstraints(checkBox, c);
            panel.add(checkBox);
            // creates and display checkbox for displaying as attribute
            JCheckBox checkBox2 = new JCheckBox();
            if (xmlDisplay.isElementAttributeLike(subElements[i])) {
                displayAsAttrBoxes.put(checkBox2, subElements[i]);
                checkBox2.setSelected(xmlDisplay.isSubElementDisplayedAsAttr(
                        name, subElements[i]));
                checkBox2.addItemListener(this);
            } else {
                checkBox2.setEnabled(false);
            }
            c.weightx = 0;
            c.gridx = 2;
            c.anchor = GridBagConstraints.CENTER;
            c.insets = new Insets(0, 10, 0, 0);
            layout.setConstraints(checkBox2, c);
            panel.add(checkBox2);
        }
    }

    /**
     * called each time an item event occurs
     * 
     * @param e the ItemEvent
     */
    public void itemStateChanged(ItemEvent e) {
        // gets the source
        JCheckBox checkBox = (JCheckBox) e.getSource();
        // looks for a 'display in tree' event
        String name = (String) displayInTreeBoxes.get(checkBox);
        // If it is one, validates the change
        if (name != null) {
            xmlDisplay.setSubElementDisplayed((String) box.getSelectedItem(),
                    name, checkBox.isSelected());
        } else {
            // else it is a 'display as attribute' event
            String name2 = (String) displayAsAttrBoxes.get(checkBox);
            xmlDisplay.setSubElementDisplayedAsAttr((String) box
                    .getSelectedItem(), name2, checkBox.isSelected());
        }
    }

}