/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.specific;

import java.io.Serializable;
import java.util.Enumeration;

import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import ch.cern.lhcb.xmleditor.gui.XmlTree;
import ch.cern.lhcb.xmleditor.gui.XmlTreeNode;
import ch.cern.lhcb.xmleditor.xerces.TextImpl;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * this class extends the default tree model and provides two new features : -
 * the ability to replace a node in the tree - the ability to add a whole
 * subtree in the tree
 * 
 * @author Sebastien Ponce
 */
public class Model extends DefaultTreeModel implements Serializable {

    /**
     * a pointer to the tree using this model
     */
    private XmlTree jTree;

    /**
     * constructor of Model
     */
    public Model() {
        // calls the ancestor's constructor
        super(new XmlTreeNode());
    }

    /**
     * sets the XmlTree owning this model
     * 
     * @param jTree a pointer to the tree using this model
     */
    public void setXmlTree(XmlTree jTree) {
        this.jTree = jTree;
    }

    /**
     * this overwrites the original isLeaf method in order to make the
     * references expandable
     * 
     * @param node the node we are dealing with
     */
    public boolean isLeaf(Object node) {
        // first calls the original method
        boolean result = super.isLeaf(node);
        // if the node is not null, look if it is a reference
        if (node != null) {
            if (((XmlTreeNode) node).isReference()) { return false; }
        }
        return result;
    }

    /**
     * changes the root node of the model
     * 
     * @param root the new root node
     */
    public void setRootNode(TreeElement root) {
        // first clears the tree
        XmlTreeNode oldRoot = (XmlTreeNode) getRoot();
        oldRoot.removeAllChildren();
        oldRoot.removeFromParent();
        // then repopulates it
        insertTree(root, null, 0);
        // validates the changes
        fireTreeStructureChanged(this, new Object[] { getRoot()}, new int[0],
                new Object[0]);
    }

    /**
     * replaces a TreeNode by a new one, created from a Node. Returns the new
     * XmlTreeNode. Usually called when a reference node is dereferenced.
     * 
     * @param node the node to be added
     * @param oldNode the old node that will be replaced
     * @return the new XmlTreeNode
     */
    public XmlTreeNode replaceNode(TreeElement node, XmlTreeNode oldNode) {
        // gets the parent node and the index of the node to replace in this
        // parent
        XmlTreeNode parent = oldNode.getParentTreeNode();
        int index = parent.getIndex(oldNode);
        // creates the new tree node
        XmlTreeNode newNode = new XmlTreeNode(node, parent.getXmlTree());
        // removes the old node and add the new one
        parent.replaceNode(newNode, index);
        nodesChanged(parent, new int[] { index});
        // inserts the children of the new node
        insertChildren(node, newNode);
        // creates a link to the oldNode in the new TreeNode and vice versa
        newNode.setOldNode(oldNode);
        oldNode.setNewNode(newNode);
        // returns the new node
        return newNode;
    }

    /**
     * insert a subtree at a given position in another one
     * 
     * @param node the root node of the subtree
     * @param parent the parent node of the subtree, once inserted
     * @param index the position of the subtree among the children of parents
     * @return the XmlTreeNode created from node
     */
    public XmlTreeNode insertTree(TreeElement node, XmlTreeNode parent,
            int index) {
        // verifies that the subtree to add is not empty
        if (node == null) { return null; }
        // deals with ignorable spaces nodes
        if (node.getNodeValue() != null) {
            if (node.getNodeType() == Node.TEXT_NODE) {
                if (((TextImpl) node).isIgnorableWhitespace()) { return null; }
            }
        }
        // gets the XmlTree in which this new node will be inserted
        // It's basically the one of its parent except if the new node will be
        // root
        XmlTree tree;
        if (parent != null) {
            tree = parent.getXmlTree();
        } else {
            tree = jTree;
        }
        // creates a new tree node
        XmlTreeNode treeNode = new XmlTreeNode(node, tree);
        // adds it at the given position. If parent is null, then it will be
        // root
        if (parent == null) {
            setRoot(treeNode);
        } else {
            insertNodeInto(treeNode, parent, index);
        }
        // inserts the children of the new node
        insertChildren(node, treeNode);
        // returns
        return treeNode;
    }

    /**
     * insert the children of a node in the subtree of the corresponding
     * TreeNode
     * 
     * @param node the node whose children will be inserted
     * @param treeNode the corresponding treeNode
     */
    private void insertChildren(TreeElement node, XmlTreeNode treeNode) {
        // iterates over the children of node
        NodeList nodes = node.getChildNodes();
        if (nodes != null) {
            // insert every node
            for (int i = 0; i < nodes.getLength(); i++) {
                // this only keeps node that can be displayed
                // it actually removes only DocumentType nodes
                if (nodes.item(i) instanceof TreeElement) {
                    TreeElement child = (TreeElement) nodes.item(i);
                    // insert it as last children
                    insertTree(child, treeNode, treeNode.getChildCount());
                }
            }
        }
    }

    /**
     * redisplays the whole tree
     */
    public void redisplayTree() {
        Enumeration expanded = jTree.getExpandedDescendants(new TreePath(
                getRoot()));
        TreePath[] selectionPaths = jTree.getSelectionPaths();
        this.reload();
        if (expanded != null) {
            while (expanded.hasMoreElements()) {
                TreePath treePath = (TreePath) expanded.nextElement();
                jTree.expandPath(treePath);
            }
        }
        jTree.setSelectionPaths(selectionPaths);
    }

    /**
     * overrides the default method. Same behavior
     * 
     * @param newChild the node to add
     * @param parent its future parent
     * @param index its future index among children of parent
     */
    public void insertNodeInto(XmlTreeNode newChild, XmlTreeNode parent,
            int index) {
        // inserts the node
        boolean displayed = parent.insertNode(newChild, index);
        // redisplays the tree only if the new node is displayed
        if (displayed) {
            nodesWereInserted(parent, new int[] { index});
        }
    }

}