/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.xerces;

import java.util.HashMap;

import org.w3c.dom.Attr;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.Entity;
import org.w3c.dom.EntityReference;
import org.w3c.dom.Text;

import ch.cern.lhcb.xmleditor.gui.XmlTreeNode;
import ch.cern.lhcb.xmleditor.specific.XmlFile;
import ch.cern.lhcb.xmleditor.undo.UndoManager;

/**
 * This class subclasses the default DocumentImpl implementation to use
 * ch.cern.lhcb.xmleditor.xerces.ElementImpl in place of regular ElementImpl
 * 
 * @author Sebastien Ponce
 */
public class DocumentImpl extends org.apache.xerces.dom.DocumentImpl implements
        TreeElement {

    /**
     * This is the XmlFile associated to the element
     */
    private XmlFile file;

    /**
     * the UndoManager associated to this element
     */
    private UndoManager undoManager;

    /**
     * A list of existing entities in order to get easily these
     */
    private HashMap entities;

    /**
     * Default Constructor Has to exist due to DOMParser limitations
     */
    public DocumentImpl() {
        super();
        this.file = null;
        undoManager = null;
        entities = new HashMap();
    }

    /**
     * constructor
     * 
     * @param doctype the type of the new document
     */
    public DocumentImpl(DocumentType doctype) {
        super(doctype);
        this.file = null;
        undoManager = null;
        entities = new HashMap();
    }

    /**
     * an accessor to the file associated with this node
     * 
     * @return the new file associated with this node
     */
    public XmlFile getXmlFile() {
        return file;
    }

    /**
     * an accessor to the file associated with this node
     * 
     * @param file the new file associated with this node
     */
    public void setXmlFile(XmlFile file) {
        this.file = file;
    }

    /**
     * gets the undoManager
     */
    public UndoManager getUndoManager() {
        return this.undoManager;
    }

    /**
     * sets the undoManager
     */
    public void setUndoManager(UndoManager undoManager) {
        this.undoManager = undoManager;
    }

    /**
     * adds treeNode to the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to add to the list
     */
    public void addTreeNode(XmlTreeNode treeNode) {
        // nothing to do since never called
    }

    /**
     * removes treeNode from the list of treeNode associated with this node
     * 
     * @param treeNode the TreeNode to remove from the list
     */
    public void removeTreeNode(XmlTreeNode treeNode) {
        // nothing to do since never called
    }

    /**
     * this method should be called whenever the value of this node is edited
     */
    public void nodeNameChanged() {
        // nothing to do since never called
    }

    /**
     * this method should be called whenever the value of this node is edited
     */
    public void nodeValueChanged() {
        // nothing to do since never called
    }

    /**
     * this method should be called when this node is removed
     */
    public void removed() {
        // nothing to do since never called
    }

    /**
     * sets the value of this node, if it has one. The value of such a node is
     * in fact the value of its text child if it exists
     * 
     * @param value the new value of this node
     */
    public void setValue(String value) {
        this.setNodeValue(value);
    }

    /**
     * gets the value of this node, if it has one. The value of such a node is
     * in fact the value of its text child if it exists
     * 
     * @return the value of this node, null it has none
     */
    public String getValue() {
        return this.getNodeValue();
    }

    /**
     * creates an Element having this Document as its OwnerDoc. Overwrites the
     * default method in order to create XmlEditor Elements
     * 
     * @param tagName The name of the element type to instantiate.
     * @throws DOMException(INVALID_NAME_ERR) if the tag name is not acceptable.
     */
    public Element createElement(String tagName) throws DOMException {
        if (errorChecking && !isXMLName(tagName, true)) { throw new DOMException(
                DOMException.INVALID_CHARACTER_ERR, "DOM002 Illegal character"); }
        ElementImpl result = new ElementImpl(this, tagName);
        return result;
    }

    /**
     * creates an Attribute having this Document as its OwnerDoc. Overwrites the
     * default method in order to create XmlEditor Attrs
     * 
     * @param name The name of the attribute.
     * @throws DOMException(INVALID_NAME_ERR) if the attribute name is not
     *         acceptable.
     */
    public Attr createAttribute(String name) throws DOMException {
        if (errorChecking && !isXMLName(name, true)) { throw new DOMException(
                DOMException.INVALID_CHARACTER_ERR, "DOM002 Illegal character"); }
        return new AttrImpl(this, name);
    }

    /**
     * creates a Text node having this Document as its OwnerDoc. Overwrites the
     * default method in order to create XmlEditor Text
     * 
     * @param data The initial contents of the Text.
     */
    public Text createTextNode(String data) {
        return new TextImpl(this, data);
    }

    /**
     * creates a Comment having this Document as its OwnerDoc. Overwrites the
     * default method in order to create XmlEditor Comment
     * 
     * @param data The initial contents of the Comment.
     */
    public Comment createComment(String data) {
        return new CommentImpl(this, data);
    }

    public EntityReference createEntityReference(String name)
            throws DOMException {
        if (errorChecking && !isXMLName(name, true)) { throw new DOMException(
                DOMException.INVALID_CHARACTER_ERR, "DOM002 Illegal character"); }
        return new org.apache.xerces.dom.EntityReferenceImpl(this, name);
    }

    public Entity createEntity(String name) throws DOMException {
        if (errorChecking && !isXMLName(name, true)) { throw new DOMException(
                DOMException.INVALID_CHARACTER_ERR, "DOM002 Illegal character"); }
        Entity entity = new org.apache.xerces.dom.EntityImpl(this, name);
        entities.put(name, entity);
        return entity;
    }

    /**
     * says whether node is an ancestor of this node or not Take care that the
     * notion of ancestor does include the parent-child relation as well as the
     * element-attribute one. This means that an element is ancestor of its
     * attributes.
     * 
     * @param node the node to be tested
     * @return true if the current node is a descendant of node
     */
    public boolean isNodeAncestor(TreeElement node) {
        if (node == this) { return true; }
        // there is no parent to test, so return false
        return false;
    }
}
