/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.plots;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.simulogic.dataview.ResourceManager;
import com.simulogic.dataview.DataChannel;

/**
 * Plot that uses styled lines in histogram form to connect data points
 */
public class HistogramPlot extends AbstractPlot {

    private GeneralPath gp = new GeneralPath();
    private Point2D.Double tp;
    private Point2D.Double ta;
    
    /**
     * Creates new HistogramPlot
     */
    public HistogramPlot() {
        super();
        distance = .5;
    }

    /**
     * Plots content of the single channel
     * 
     * @param g2 graphics
     * @param xChannel channel containing independent variable values
     * @param yChannel channel containing dependent variable values
     * @param xMin minimal value for plotting range
     * @param xMax maximal value for plotting range
     * @param transform plot transform
     */
    public final void plotChannel(Graphics2D g2, DataChannel xChannel,
            DataChannel yChannel, double xMin, double xMax,
            AffineTransform transform) {
        try {
            if (yChannel.getAttribute().isNormal()) {
                g2.setColor(yChannel.getAttribute().getColor());
                g2.setStroke(yChannel.getAttribute().getStroke());
                
                tp = new Point2D.Double();
                ta = new Point2D.Double();
                AffineTransform d_tr = transform.createInverse();
                tp .setLocation(0, 0);
                d_tr.transform(tp, tp);
                ta .setLocation(distance, 0);
                d_tr.transform(ta, ta);
                double dist_x = ta.distance(tp);
                ta .setLocation(0, distance);
                d_tr.transform(ta, ta);
                double dist_y = ta.distance(tp);
                
                gp.reset();
                tp = null;
                double xp = 0;
                double yp = 0;

                for (int k = 0; k < xChannel.size(); k++) {
                    double xval = xChannel.getData(k);
                    if ((xval >= xMin) && (xval <= xMax)
                            && (k < yChannel.size())) {
                        double yval = yChannel.getData(k);
                        if (tp == null) {
                            tp = new Point2D.Double(xval, yval);
                            ta = new Point2D.Double(xval, yval);
                            gp.moveTo((float) xval, (float) yval);
                            xp = xval;
                            yp = yval;
                        } else {
                            tp.setLocation(xval, yval);

                            if ((Math.abs(ta.x-tp.x) >= dist_x) || (Math.abs(ta.y-tp.y) >= dist_y)) {
                                double mid = xp + Math.abs((xval-xp)/2);
                                gp.lineTo((float) mid, (float) yp);
                                gp.lineTo((float) mid, (float) yval);
                                gp.lineTo((float) xval, (float) yval);
                                ta.setLocation(xval, yval);
                                xp = xval;
                                yp = yval;
                            }
                        }
                    }
                }
                g2.draw(gp.createTransformedShape(transform));
            }
        } catch (Exception e) {
        }
    }

    /**
     * Determines whether point is located within a plot
     * 
     * @param x point x coordinate
     * @param y point y coordinate
     * @param xChannel channel containing independent variable values
     * @param yChannel channel containing dependent variable values
     * @param transform plot transform
     * 
     * @return true if point belongs to channel
     */
    public boolean isChannelPoint(double x, double y, DataChannel xChannel,
            DataChannel yChannel, AffineTransform transform) {
        Point2D.Double dstTop = new Point2D.Double();
        Point2D.Double dstBtm = new Point2D.Double();

        try {
            transform.inverseTransform(new Point2D.Double(x, y - hotspot),
                    dstTop);

            transform.inverseTransform(new Point2D.Double(x, y + hotspot),
                    dstBtm);

            double tst = com.simulogic.dataview.analysis.ChannelStats
                    .getValueAtX(dstTop.getX(), xChannel, yChannel);

            if ((tst > dstBtm.getY()) && (tst < dstTop.getY())) { return true; }
        } catch (Exception e) {
        }
        return false;
    }

    /**
     * Returns new instance of the plot
     * 
     * @return new instance of the current plot
     */
    public AbstractPlot createPlot() {
        return new HistogramPlot();
    }
    
    /**
     * Returns plot name.
     * 
     * @return plot name.
     */
    public String getName() {
    	return ResourceManager.getResource("Histogram_plot");
    }
    
    /**
     * Returns small icon for plot.
     * 
     * @return mall icon for plot.
     */
    public Icon getSmallIcon() {
    	return new ImageIcon(this.getClass().getResource("histogramplot_image"));
    }
}